/*
 *  Copyright 2001 Adrian Thurston <adriant@ragel.ca>
 */

/*  This file is part of Aapl.
 *
 *  Aapl is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Aapl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Aapl; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

#ifndef _AAPL_COMPARE_H
#define _AAPL_COMPARE_H

#include "table.h"
#include "string.h"

/*
 * StringCompare 
 */
struct StringCompare
{
	static inline int Compare(const char *k1, const char *k2)
		{ return strcmp(k1, k2); }
};

/*
 * OrdinalCompare 
 */
template<class T> struct OrdinalCompare
{
	static inline int Compare(const T k1, const T k2)
	{
		if (k1 < k2)
			return -1;
		else if (k1 > k2)
			return 1;
		else
			return 0;
	}
};

/*
 * TableCompare
 */
template<class T, class CompareClass> struct TableCompare
{
	static inline int Compare(const BaseTable<T> &t1, const BaseTable<T> &t2)
	{
		if ( t1.TableLength < t2.TableLength )
			return -1;
		else if ( t1.TableLength > t2.TableLength )
			return 1;
		else
		{
			T *i1 = t1.Table, *i2 = t2.Table;
			int len = t1.TableLength, cmpResult;
			for ( int pos = 0; pos < len;
					pos += 1, i1 += 1, i2 += 1 )
			{
				cmpResult = CompareClass::Compare(*i1, *i2);
				if ( cmpResult != 0 )
					return cmpResult;
			}
			return 0;
		}
	}
};

#endif /* _AAPL_COMPARE_H */
