/*
 *  Copyright 2001 Adrian Thurston <adriant@ragel.ca>
 */

/*  This file is part of Aapl.
 *
 *  Aapl is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Aapl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Aapl; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

#include <stdio.h>
#include <iostream>
#include <vector>
#include "vector.h"

void testVector1()
{
	int pos, len;
	char c, string[200];
	Vector< char, LinearTable<char> > v;

	while( scanf("%c", &c) == 1 )
	{
		switch (c) {
			case 'a':
				/* Get the string to append. */
				fgets(string, 200, stdin);

				/* hack off the newline. */
				len = strlen(string) - 1;
				string[len] = 0;

				v.Append(string, len);
				break;

			case 's':
				/* Get the string to set. */
				fgets(string, 200, stdin);

				/* hack off the newline. */
				len = strlen(string) - 1;
				string[len] = 0;

				v.SetAs(string, len);
				break;

			case 'o':
				/* Get the pos */
				scanf("%i ", &pos);

				/* Get the string to overwrite. */
				fgets(string, 200, stdin);

				/* hack off the newline. */
				len = strlen(string) - 1;
				string[len] = 0;

				v.Overwrite(pos, string, len);
				break;
				
			case 'i':
				/* Get the pos */
				scanf("%i ", &pos);

				/* Get the string to insert. */
				fgets(string, 200, stdin);

				/* hack off the newline. */
				len = strlen(string) - 1;
				string[len] = 0;

				v.Insert(pos, string, len);
				break;
				
			case 'd':
				/* Get the length to delete. */
				scanf("%i", &pos);
				scanf("%i", &len);
				v.Delete(pos, len);
				break;

			case 'p':
				printf("L: %i  AL: %i\n", v.TableLength, v.AllocatedLength);
				fwrite(v.Table, 1, v.TableLength, stdout);
				fputc('\n', stdout);
				break;

			case 'q':
				goto end;

		}
	}

end:
	fwrite(v.Table, 1, v.TableLength, stdout);
	fputc('\n', stdout);
}


int defaultCount = 0, copyCount = 0, destructorCount = 0;

struct theData {
	theData() : i(0) { defaultCount++; }
	theData(const theData &d) : i(1) { copyCount++; }
	~theData() { destructorCount++; }
	int i;
};

static int returnValue = 0;
void assertion( int expr )
{
	if ( ! expr ) {
		cerr << "testVector: test failed!" << endl;
		returnValue = 1;
	}
	defaultCount = copyCount = destructorCount = 0;
}

void testDelete()
{
	/* Simple Construct calls. */
	theData sampleData[10];
	assertion( defaultCount == 10 );
	
	/* Initialization. */
	Vector< theData, LinearTable<theData> > v1(10);
	assertion( defaultCount == 10 );

	/* Delete some. */
	v1.Delete( 5, 3 );
	assertion( destructorCount == 3 );

	/* Delete some more. */
	v1.Delete( 0, 4 );
	assertion( destructorCount == 4 );

	/* Test length. */
	assertion( v1.TableLength == 3 );
}
void testInsert()
{
	/* Simple Construct calls. */
	theData sampleData[10];
	assertion( defaultCount == 10 );
	
	/* Initialization. */
	Vector< theData, LinearTable<theData> > v1(6);
	assertion( defaultCount == 6 );

	/* Copy Constructor. */
	Vector< theData, LinearTable<theData> > v2(v1);
	assertion( copyCount == 6 );

	/* Insert, some shifted over. */
	v1.Insert(1, sampleData, 4);
	assertion( v1.TableLength == 10 && destructorCount == 0 && copyCount == 4 );

	/* Insert, none shifted over. */
	v1.Insert(10, sampleData, 4);
	assertion( defaultCount == 0 && copyCount == 4 );

	/* Insert, none shifted over, some constructors called. */
	v1.Insert(20, sampleData, 5);
	assertion( destructorCount == 0 && copyCount == 5 && defaultCount == 6 );

	/* Ensure length is good. */
	assertion( v1.TableLength == 25 );
}

void testSetAs()
{
	/* Simple Construct calls. */
	theData sampleData[10];
	assertion( defaultCount == 10 );
	
	/* Initialization. */
	Vector< theData, LinearTable<theData> > v1(6);
	assertion( defaultCount == 6 );

	v1.SetAs( sampleData, 10 );
	assertion( destructorCount == 6 && copyCount == 10 && defaultCount == 0 );
}

void testOverwrite()
{
	/* Simple Construct calls. */
	theData sampleData[10];
	assertion( defaultCount == 10 );
	
	/* Initialization. */
	Vector< theData, LinearTable<theData> > v1(4);
	assertion( defaultCount == 4 );

	/* Copy Constructor. */
	Vector< theData, LinearTable<theData> > v2(v1);
	assertion( copyCount == 4 );

	/* Overwrite, some delete, some new. */
	v1.Overwrite(1, sampleData, 4);
	assertion( destructorCount == 3 && copyCount == 4 );

	/* Overwrite, all delete. */
	v1.Overwrite(0, sampleData, 5);
	assertion( destructorCount == 5 && copyCount == 5 );

	/* Overwrite, all new. */
	v1.Overwrite(5, sampleData, 2);
	assertion( destructorCount == 0 && copyCount == 2 );

	/* Overwrite, all new, some must be initted. */
	v1.Overwrite(10, sampleData, 2);
	assertion( destructorCount == 0 && defaultCount == 3 && copyCount == 2 );

	/* Ensure length is good. */
	assertion( v1.TableLength == 12 );
}

void testWithValues()
{
	Vector< int, LinearTable<int> > v1;
	Vector< int, LinearTable<int> > v2;

	int i = 1;
	v1.Append( i );
	v2.SetAs(v1);
	i = 2;
	v1.Insert( 0, i );
	assertion( v1.Table[0] == 2 && v2.Table[0] == 1 );
}

int main() {
	testOverwrite();
	testInsert();
	testSetAs();
	testDelete();
	testWithValues();
	return returnValue;
}
