/*********************************************************
 * A mini c-like language parser.
 */

#include <stdio.h>

#define IDENT_BUFLEN 256

%% Clang
	struct {
		char identBuf[IDENT_BUFLEN+1];
		int identLen;
		int curLine;
	};
	init {
		fsm->identLen = 0;
		fsm->curLine = 1;
	}

	# Function to buffer a character.
	func bufChar {
		if ( fsm->identLen < IDENT_BUFLEN ) {
			fsm->identBuf[fsm->identLen] = c;
			fsm->identLen += 1;
		}
	}
	func clearBuf {
		fsm->identLen = 0;
	}

	func ident {
		fsm->identBuf[fsm->identLen] = 0;
		printf("ident: %s\n", fsm->identBuf);
	}
	func literal {
		fsm->identBuf[fsm->identLen] = 0;
		printf("literal: \"%s\"\n", fsm->identBuf);
	}

	func float {
		fsm->identBuf[fsm->identLen] = 0;
		printf("float: %s\n", fsm->identBuf);
	}
	func int {
		fsm->identBuf[fsm->identLen] = 0;
		printf("int: %s\n", fsm->identBuf);
	}
	func hex {
		fsm->identBuf[fsm->identLen] = 0;
		printf("hex: %s\n", fsm->identBuf);
	}
	func symbol {
		fsm->identBuf[fsm->identLen] = 0;
		printf("symbol: %s\n", fsm->identBuf);
	}

	alnumu = alnum | '_';
	alphau = alpha | '_';

	symbol = ( punct - [_] ) >clearBuf $bufChar %symbol;

	ident = (alphau . *alnumu) >clearBuf $bufChar %ident;

	literalChar = 
			( extend - ['"\\] ) :bufChar |
			( '\\' . extend :bufChar );

	sliteral = ('\'' :clearBuf . *literalChar . '\'' ) %literal;
	dliteral = ('"' :clearBuf . *literalChar . '"' ) %literal;

	literal = sliteral | dliteral;

	whitespace = space | '\n' | 0x0..0x31 | 0x7f..0xff;

	ccComment = '//' . *extend . '\n' :1;
	cComment = '/*' . *extend . '*/' :1;

	int = +digit >clearBuf $bufChar %int;
	float =  ( +digit . '.' . +digit ) >clearBuf $bufChar %float;
	hex = ('0x' . +xdigit) >clearBuf $bufChar %hex;

	fin = ( ccComment |
		cComment |
		symbol |
		ident |
		literal |
		whitespace |
		int |
		float |
		hex );

	clang_main = *( fin % -1 );

	newline = extend | '\n' : { fsm->curLine += 1; };

	main = 
		clang_main $0 %! | 
		newline;
%%

#define BUFSIZE 2048

int main()
{
	char buf[BUFSIZE];
	Clang clang;

	ClangInit( &clang );
	while ( 1 ) {
		int len = fread( buf, 1, BUFSIZE, stdin );

		ClangExecute( &clang, buf, len );

		if ( len != BUFSIZE )
			break;
	}
	ClangFinish( &clang );
	return 0;
}

