/**************************************************************
 * Parses unix mail boxes into headers and bodies.
 */

#include <stdio.h>
#define LINEBUF 1024
#define BUFSIZE 2048

%% MailboxScanner
	struct {
		char lineBuf[LINEBUF];
		int lineLen;

		/* Write out the line buffer with some prefix. */
		void WriteLine( char *pref );
	};
%%

void MailboxScanner::WriteLine( char *pref )
{
	fprintf( stdout, "%s: ", pref );
	fwrite( lineBuf, 1, lineLen, stdout );
	lineLen = 0;
}

%% MailboxScanner

	init {
		lineLen = 0;
	}

	postfunc {
		if ( lineLen < LINEBUF ) {
			lineBuf[lineLen] = c;
			lineLen += 1;
		}
	}

	func fromln { WriteLine( "fromln" ); }
	func blankl { WriteLine( "blankl" ); }
	func header { WriteLine( "header" ); }
	func hdcont { WriteLine( "hdcont" ); }
	func msglin { WriteLine( "msglin" ); }

	simpleLine = *( extend - '\n' ) . '\n';

	# After seeing the 'From ' we bump up the priority so that
	# we are sure to leave the previous messages's body.

	day = /[A-Z][a-z][a-z]/;
	month = /[A-Z][a-z][a-z]/;
	year = /[0-9][0-9][0-9][0-9]/;
	time = /[0-9][0-9]:[0-9][0-9]/ . ( /:[0-9][0-9]/ | '' );
	letterZone = /[A-Z][A-Z][A-Z]/;
	numZone = /[+-][0-9][0-9][0-9][0-9]/;
	zone = letterZone | numZone;
	dayNum = /[0-9 ][0-9]/;

	date = day . ' ' . month . ' ' . dayNum . ' ' . time . ' ' .
		( year | year . ' ' . zone | zone . ' ' . year );

	fromLine = 'From ' . *(!'\n') . ' ' . date . '\n' :+1 %fromln;

	hchar = print - [ :];

	header = +hchar . ':' .
		*( extend - '\n' | '\n'. [ \t] :hdcont ) . '\n' %header;

	messageLine = simpleLine %msglin;

	# When we get to the last newline we are still matching messageLine
	# so on the last newline it will think we are still in the message.
	# We need this because we can't assume that every newline means
	# the end of the current message, whereas at the same time we requre
	# that there be a newline before the fromLine of the next message.
	message = ( fromLine .  *header .  '\n' %blankl .  *messageLine .^ '\n' );

	# Its important that the priority in the fromLine gets bumped up
	# so that we are able to move to new messages. Otherwise we
	# will always stay in the message body.
	main = ( *message );

%%

MailboxScanner mailbox;
char buf[BUFSIZE];

int main()
{
	mailbox.Init();
	while ( 1 ) {
		int len = fread( buf, 1, BUFSIZE, stdin );
		mailbox.Execute( buf, len );
		if ( len != BUFSIZE )
			break;
	}
	mailbox.Finish();
	if ( ! mailbox.Accept() )
		printf("*** fail ***\n");
	return 0;
}
